<?php

namespace Drupal\crossword_image\Plugin\crossword\crossword_image;

use Drupal\file\FileInterface;

/**
 * This makes an image of the blank grid with numbers.
 *
 * It's a little bigger in pixels and memory than
 * the plain thumbnail.
 *
 * @CrosswordImage(
 *   id = "numbered_thumbnail",
 *   title = @Translation("Numbered Thumbnail"),
 *   type = IMAGETYPE_PNG,
 *   toolkit = "gd"
 * )
 */
class CrosswordNumberedThumbnail extends CrosswordThumbnailBase {

  /**
   * {@inheritdoc}
   */
  protected $squareSize = 40;

  /**
   * {@inheritdoc}
   */
  protected $lineSize = 3;

  /**
   * The font to be used by imagestring().
   *
   * @var int
   *
   * @see https://www.php.net/manual/en/function.imagestring.php
   */
  protected $font = 3;

  /**
   * {@inheritdoc}
   */
  public function createImageResource(FileInterface $file) {
    $image = parent::createImageResource($file);
    $data = $this->crosswordDataService->getData($file);
    $grid = $data['puzzle']['grid'];
    $square_size = $this->squareSize;
    $line_size = $this->lineSize;
    $black = imagecolorallocate($image, 0, 0, 0);
    foreach ($grid as $row_index => $row) {
      foreach ($row as $col_index => $square) {
        if (isset($square['numeral'])) {
          $color = $black;
          imagestring($image, $this->font, $col_index * $square_size + $line_size / 2 + $line_size, $row_index * $square_size + $line_size / 2, $square['numeral'], $color);
        }
        if (!empty($square['circle'])) {
          $start = isset($square['numeral']) ? -90 : -180;
          $end = 180;
          $color = $black;
          $height = $width = $this->squareSize;
          $x = ($col_index + 0.5) * $square_size;
          $y = ($row_index + 0.5) * $square_size;
          // Draw two arcs to make the line thicker.
          imagearc($image, $x, $y, $width, $height, $start, $end, $color);
          imagearc($image, $x, $y, $width - 1, $height - 1, $start, $end, $color);
        }
      }
    }
    return $image;
  }

}
