<?php

namespace Drupal\crossword;

use Drupal\file\FileInterface;

/**
 * Interface for CrosswordDataService.
 */
interface CrosswordDataServiceInterface {

  /**
   * Get structured crossword data from a file.
   *
   * See CrosswordFilePluginInterface::parse() for returned data structure.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   The structured data.
   */
  public function getData(FileInterface $file);

  /**
   * Get the crossword author from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   Return author if available.
   */
  public function getAuthor(FileInterface $file);

  /**
   * Get the crossword dimension from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   Return number of columns if available.
   */
  public function getDimensionAcross(FileInterface $file);

  /**
   * Get the crossword dimension from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   Return number of rows if available.
   */
  public function getDimensionDown(FileInterface $file);

  /**
   * Get the crossword dimensions from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   * @param string $delimiter
   *   The delimiter.
   *
   * @return mixed
   *   Return number of dimensions if available.
   */
  public function getDimensions(FileInterface $file, $delimiter = 'x');

  /**
   * Get the crossword title from a file.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return mixed
   *   Return title of crossword if available. (Not the file name!)
   */
  public function getTitle(FileInterface $file);

  /**
   * Find out if it's a rebus puzzle, which may require more instructions.
   *
   * @param Drupal\file\FileInterface $file
   *   The (crossword) file.
   *
   * @return bool
   *   True if it's a rebus crossword puzzle.
   */
  public function isRebus(FileInterface $file);

}
