<?php

namespace Drupal\crossword\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'crossword_screenreader' formatter.
 *
 * @FieldFormatter(
 *   id = "crossword_screenreader",
 *   label = @Translation("Crossword Puzzle (screenreader)"),
 *   weight = "2",
 *   field_types = {
 *     "crossword"
 *   }
 * )
 */
class CrosswordFormatterScreenreader extends CrosswordFormatter {

  const BUTTONS = [
    'solution' => 'Solution',
    'clear' => 'Clear',
  ];

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $options['details'] = [
      'title_tag' => 'h1',
      'author_tag' => 'h2',
      'notepad_tag' => 'p',
    ];
    $options['buttons']['buttons'] = [];
    foreach (CrosswordFormatterScreenreader::BUTTONS as $key => $label) {
      $options['buttons']['buttons'][$key] = [
        'show' => TRUE,
        'input_label' => $label,
      ];
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $form = parent::settingsForm($form, $form_state);
    unset($form['print']);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $file) {

      $data = $this->crosswordDataService->getData($file);

      $elements[$delta] = [
        '#theme' => 'crossword',
        '#screenreader' => TRUE,
        '#content' => [
          'title' => $this->getTitle($data),
          'author' => $this->getAuthor($data),
          'notepad' => $this->getNotepad($data),
          'congrats' => $this->getSetting('congrats'),
          'grid' => $this->getGrid($data),
          'across' => $this->getAcross($data),
          'down' => $this->getDown($data),
          'controls' => $this->getControls($file),
        ],
        '#attached' => [
          'library' => [
            'crossword/crossword.screenreader',
          ],
          'drupalSettings' => [
            'crossword' => [
              'data' => $data,
              'selector' => '.crossword',
            ],
          ],
        ],
        '#cache' => [
          'tags' => $file->getCacheTags(),
        ],
      ];
      // We don't need or want the rebus checkbox.
      unset($elements[$delta]['#content']['controls']['#content']['rebus']);
    }
    return $elements;
  }

}
