<?php

namespace Drupal\crossword\Plugin\Validation\Constraint;

use Drupal\Core\Entity\EntityTypeManager;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\crossword\CrosswordFileParserManagerInterface;

/**
 * Validates the crossword file.
 */
class CrosswordFileValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * Crossword Parser manager service.
   *
   * @var \Drupal\crossword\CrosswordFileParserManagerInterface
   */
  protected $parserManager;

  /**
   * File storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $fileStorage;

  /**
   * Create an instance of the validator.
   *
   * @param \Drupal\crossword\CrosswordFileParserManagerInterface $parser_manager
   *   The parser manager service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   *   Entity type manager.
   */
  public function __construct(CrosswordFileParserManagerInterface $parser_manager, EntityTypeManager $entity_type_manager) {
    $this->parserManager = $parser_manager;
    $this->fileStorage = $entity_type_manager->getStorage('file');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('crossword.manager.parser'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function validate($items, Constraint $constraint) {

    $allowed_parsers = $items->getFieldDefinition()->getSetting('allowed_parsers');
    $allowed_parser_definitions = $this->parserManager->loadDefinitionsFromOptionList($allowed_parsers);

    foreach ($items as $item) {
      if (get_class($item) == "Drupal\Core\TypedData\Plugin\DataType\IntegerData") {
        $file = $this->fileStorage->load($item->getCastedValue());
        if (FALSE === $this->parserManager->filterApplicableDefinitions($allowed_parser_definitions, $file)) {
          $this->context->addViolation($constraint->noParser);
        }
      }
    }
  }

}
