<?php

namespace Drupal\Tests\crossword\FunctionalJavascript;

/**
 * Tests basic functionality of crossword formatter.
 *
 * @group crossword
 */
class CrosswordFormatterTest extends CrosswordFormatterTestBase {

  /**
   * {@inheritdoc}
   */
  protected $formatter = 'crossword';

  /**
   * {@inheritdoc}
   */
  public static $modules = ['crossword_tests'];

  /**
   * {@inheritdoc}
   */
  protected $testsActiveClues = TRUE;

  /**
   * Tests clues toggle.
   */
  public function testCluesToggle() {
    $node = $this->createTestNode();
    $this->assertEqual(1, $node->id());

    // View the crossword node at its normal path.
    // Allows us to test non-default config.
    $this->drupalGet('node/1');
    $assertSession = $this->assertSession();
    $session = $this->getSession();
    $page = $session->getPage();

    // By default, there is no clue toggling and clues are visible.
    $assertSession->elementNotExists('css', '#show-clues');
    $assertSession->elementExists('css', '.crossword-clues.across');
    $assertSession->elementExists('css', '.crossword-clues.down');

    // Add the toggle button, initially unchecked.
    $config = \Drupal::service('config.factory')->getEditable('core.entity_view_display.node.crossword.default');
    $config->set('content.field_crossword.settings.clues.show', TRUE);
    $config->save();
    drupal_flush_all_caches();

    $this->drupalGet('node/1');
    $assertSession->elementExists('css', '#show-clues');
    $assertSession->elementNotExists('css', '#show-clues:checked');
    $this->assertFalse($page->find('css', '.crossword-clues.across')->isVisible());
    $this->assertFalse($page->find('css', '.crossword-clues.down')->isVisible());
    // Click the toggle button to show clues.
    $page->find('css', '#show-clues')->click();
    $session->wait(20);
    $this->assertTrue($page->find('css', '.crossword-clues.across')->isVisible());
    $this->assertTrue($page->find('css', '.crossword-clues.down')->isVisible());

    // Add the toggle button, initially checked.
    $config = \Drupal::service('config.factory')->getEditable('core.entity_view_display.node.crossword.default');
    $config->set('content.field_crossword.settings.clues.checked', TRUE);
    $config->save();
    drupal_flush_all_caches();

    $this->drupalGet('node/1');
    $assertSession->elementExists('css', '#show-clues');
    $assertSession->elementExists('css', '#show-clues:checked');
    $this->assertTrue($page->find('css', '.crossword-clues.across')->isVisible());
    $this->assertTrue($page->find('css', '.crossword-clues.down')->isVisible());
    // Click the toggle button to show clues.
    $page->find('css', '#show-clues')->click();
    $session->wait(20);
    $this->assertFalse($page->find('css', '.crossword-clues.across')->isVisible());
    $this->assertFalse($page->find('css', '.crossword-clues.down')->isVisible());

    // Do not render clues. The button should not render either.
    $config = \Drupal::service('config.factory')->getEditable('core.entity_view_display.node.crossword.default');
    $config->set('content.field_crossword.settings.clues.do_not_render', TRUE);
    $config->save();
    drupal_flush_all_caches();

    $this->drupalGet('node/1');
    $assertSession->elementNotExists('css', '#show-clues');
    $assertSession->elementNotExists('css', '.crossword-clues.across');
    $assertSession->elementNotExists('css', '.crossword-clues.down');
  }
}
