<?php

namespace Drupal\Tests\crossword\FunctionalJavascript;

/**
 * Tests basic functionality of crossword_screenreader formatter.
 *
 * @group crossword
 */
class CrosswordScreenreaderFormatterTest extends CrosswordFormatterTestBase {

  /**
   * {@inheritdoc}
   */
  protected $formatter = 'crossword_screenreader';

  /**
   * {@inheritdoc}
   */
  public static $modules = ['crossword_tests'];

  /**
   * {@inheritdoc}
   */
  protected $testsActiveClues = FALSE;

  /**
   * {@inheritdoc}
   *
   * This is very different because the js library is different.
   */
  public function testCrosswordFormatter() {

    $node = $this->createTestNode();
    $this->assertEqual(1, $node->id());

    // View the crossword node.
    $this->drupalGet("crossword-tests/1/{$this->formatter}");
    $assertSession = $this->assertSession();
    $session = $this->getSession();
    $page = $session->getPage();

    // Check for presence of various test.
    $assertSession->elementTextContains('css', '.crossword-title', "Test Puzzle");
    $assertSession->elementTextContains('css', '.crossword-author', "Test Author");

    $session->wait(20);
    // Mess with first clue.
    $label = $page->find('css', 'label[for="3-across"]');
    $expected_aria_label = '3 across. 3A Second square has a circle. 3 letters.';
    $this->assertEqual($label->getAttribute('aria-label'), $expected_aria_label);
    $input = $page->find('css', 'input[name="3-across"]');
    $text = 'bcd';
    $input->setValue($text);
    $form = $input->getParent();
    $form->submit();
    $session->wait(20);
    $expected_aria_label = '3 across. 3A Second square has a circle. Answer: 3 letters. b c d';
    $this->assertEqual($label->getAttribute('aria-label'), $expected_aria_label);
    $crossing_label = $page->find('css', 'label[for="1-down"]');
    $expected_aria_label = '1 down. 1D is AB2. Answer: 3 letters. blank b blank';
    $this->assertEqual($crossing_label->getAttribute('aria-label'), $expected_aria_label);

    // Check rebus functionality using 2-Down.
    $label = $page->find('css', 'label[for="2-down"]');
    $expected_aria_label = '2 down. 2D is 1DF. Answer: 3 letters. blank d blank';
    $this->assertEqual($label->getAttribute('aria-label'), $expected_aria_label);
    $input = $page->find('css', 'input[name="2-down"]');
    $text = 'ONEdf';
    $input->setValue($text);
    $form = $input->getParent();
    $form->submit();
    $session->wait(20);
    $expected_aria_label = '2 down. 2D is 1DF. Answer: 3 letters. one d f';
    $this->assertEqual($label->getAttribute('aria-label'), $expected_aria_label);

    // Check that the grid shows the right letters.
    $assertSession->elementTextNotContains('css', '[data-col="0"][data-row="0"] .square-fill', 'A');
    $assertSession->elementTextContains('css', '[data-col="2"][data-row="0"] .square-fill', 'ONE');
    $assertSession->elementTextContains('css', '[data-col="0"][data-row="1"] .square-fill', 'B');
    $assertSession->elementTextContains('css', '[data-col="1"][data-row="1"] .square-fill', 'C');
    $assertSession->elementTextContains('css', '[data-col="2"][data-row="1"] .square-fill', 'D');
    $assertSession->elementTextNotContains('css', '[data-col="0"][data-row="2"] .square-fill', 'TWO');
    $assertSession->elementTextNotContains('css', '[data-col="1"][data-row="2"] .square-fill', 'E');
    $assertSession->elementTextContains('css', '[data-col="2"][data-row="2"] .square-fill', 'F');

    // Check that the blank count button works.
    $button = $page->find('css', '#crossword-blank-count');
    $button->focus();
    $session->wait(20);
    $expected_aria_label = 'There are 3 blank squares. Hit return to go back to the first clue.';
    $this->assertEqual($button->getAttribute('aria-label'), $expected_aria_label);
    $button->click();
    $session->wait(20);
    // The first clue input should get focus, clearing the value.
    $input = $page->find('css', 'input[name="3-across"]');
    $this->assertEqual($input->getValue(), '');

    // Check local storage.
    $this->drupalGet("crossword-tests/1/{$this->formatter}");
    $assertSession = $this->assertSession();
    $session = $this->getSession();
    $page = $session->getPage();
    $session->wait(20);
    $assertSession->elementTextNotContains('css', '[data-col="0"][data-row="0"] .square-fill', 'A');
    $assertSession->elementTextContains('css', '[data-col="2"][data-row="0"] .square-fill', 'ONE');
    $assertSession->elementTextContains('css', '[data-col="0"][data-row="1"] .square-fill', 'B');
    $assertSession->elementTextContains('css', '[data-col="1"][data-row="1"] .square-fill', 'C');
    $assertSession->elementTextContains('css', '[data-col="2"][data-row="1"] .square-fill', 'D');
    $assertSession->elementTextNotContains('css', '[data-col="0"][data-row="2"] .square-fill', 'TWO');
    $assertSession->elementTextNotContains('css', '[data-col="1"][data-row="2"] .square-fill', 'E');
    $assertSession->elementTextContains('css', '[data-col="2"][data-row="2"] .square-fill', 'F');

    // Check the clear button.
    $page->find('css', '.button-clear')->click();
    $session->wait(20);
    $assertSession->elementTextNotContains('css', '[data-col="0"][data-row="0"] .square-fill', 'A');
    $assertSession->elementTextNotContains('css', '[data-col="2"][data-row="0"] .square-fill', 'ONE');
    $assertSession->elementTextNotContains('css', '[data-col="0"][data-row="1"] .square-fill', 'B');
    $assertSession->elementTextNotContains('css', '[data-col="1"][data-row="1"] .square-fill', 'C');
    $assertSession->elementTextNotContains('css', '[data-col="2"][data-row="1"] .square-fill', 'D');
    $assertSession->elementTextNotContains('css', '[data-col="0"][data-row="2"] .square-fill', 'TWO');
    $assertSession->elementTextNotContains('css', '[data-col="1"][data-row="2"] .square-fill', 'E');
    $assertSession->elementTextNotContains('css', '[data-col="2"][data-row="2"] .square-fill', 'F');

    // Check solution button. and the solution message.
    $message = $page->find('css', '#solution-message');
    $this->assertEqual($message->getAttribute('aria-hidden'), 'true');
    $page->find('css', '.button-solution')->click();
    $session->wait(20);
    $assertSession->elementTextContains('css', '[data-col="0"][data-row="0"] .square-fill', 'A');
    $assertSession->elementTextContains('css', '[data-col="2"][data-row="0"] .square-fill', 'ONE');
    $assertSession->elementTextContains('css', '[data-col="0"][data-row="1"] .square-fill', 'B');
    $assertSession->elementTextContains('css', '[data-col="1"][data-row="1"] .square-fill', 'C');
    $assertSession->elementTextContains('css', '[data-col="2"][data-row="1"] .square-fill', 'D');
    $assertSession->elementTextContains('css', '[data-col="0"][data-row="2"] .square-fill', 'TWO');
    $assertSession->elementTextContains('css', '[data-col="1"][data-row="2"] .square-fill', 'E');
    $assertSession->elementTextContains('css', '[data-col="2"][data-row="2"] .square-fill', 'F');
    $this->assertEqual($message->getAttribute('aria-hidden'), 'false');
  }

}
